(* mdraw.ml -- Main runner file *)

open Parser
open Scanner

type action =  Scanner | Ast | Sast | Compile

let _ =
	(* Read argument and assign to "action" *)
  let action = if Array.length Sys.argv > 1 then
    List.assoc Sys.argv.(1) [ ("-a", Ast);
                              ("-s", Scanner);
                              ("-c", Compile);
                            ]
	(* User "Scanner" by default if no argument is provided *)
  else Scanner in                             
	
  match action with
    Scanner -> let lexbuf = Lexing.from_channel stdin in
               let rec loop token  =
                  (match token with
                     | EOF -> []
                     | _ as t -> t::loop (Scanner.token lexbuf)) in
               let tokens = loop (Scanner.token lexbuf) in
               let output = Scanner_test.string_of_tokens tokens in
               print_string output
    | Ast ->   let lexbuf = Lexing.from_channel stdin in
               let abstract_syntax_tree = Parser.program Scanner.token lexbuf in
               let output = Parser_test.string_of_program abstract_syntax_tree in
               print_string output
    | Sast -> 
               let lexbuf = Lexing.from_channel stdin in
               let abstract_syntax_tree = Parser.program Scanner.token lexbuf in
               let sastree = Translate.translate abstract_syntax_tree in
               let output = Translate_test.string_of_sprogram sastree in
               print_string output 
    | Compile ->
               let lexbuf = Lexing.from_channel stdin in
               let abstract_syntax_tree = Parser.program Scanner.token lexbuf in
               let sastree = Translate.translate abstract_syntax_tree in
               let output = Compile.string_of_class (Sys.argv.(2)) sastree in
               print_string output
