type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq

type data_type =
     Int
   | Point
   | Line
   | Curve
   (*| Layer  *)
   | String
   | Boolean
   | Void
   | Ellipse


type expr =
    IntLiteral of int
  | StringLiteral of string
  | Id of string
  | MPoint of expr * expr
  | MLine of expr * expr * expr * expr 
  | MEllipse of expr * expr * expr * expr
  | MCurve of expr * expr * expr * expr * expr * expr 
  | Binop of expr * op * expr
  | Assign of expr * expr
  | Call of string * expr list
  | GetX of expr
  | GetY of expr
  (*| Layer of expr list*)

(*type and name of a variable*)
type varSig = VarSig of data_type * string

(*initialization of variable*)
type init =
  | NullInit of data_type * string   (*assign null/default value*)
  | VarInit of data_type * string * expr   (*assgin a specific value to the new variable*)


(*statement declairation*)
type stmt =
  | Block of stmt list                  (* LBRACE Statement_list RBRACE *)  
  | Expr of expr                        (* expr SEMICOLON *)
  | Return of expr
  | If of expr * stmt * stmt            (* IF LPAREN expr RPAREN Statement NOELSE SEMICOLON *)
  | For of expr * expr * expr * stmt    (* FOR LPAREN expr SEMICOLON expr SEMICOLON expr Statement SEMICOLON *)
  | While of expr * stmt                (* WHILE LPAREN expr RPAREN Statement SEMICOLON *)
  | Print of expr
  | NoElseMark
  | VarDecl of init 
  | Draw of expr


(*function declairation*)
type func_decl = {
   return : data_type;
   fname : string;
   formals : varSig list;
   body : stmt;
}


(* Bascially, the parts of a program *)
type construct = 
    GlobalVar of init
  | FuncDef of func_decl


type program = construct list (* global vars, funcs *)
