{ open Parser}  (* Get the token types *)

rule token = parse
	[' ' '\t' '\r' '\n'] { token lexbuf }  (* Whitespace *)
| "/*"     { comment lexbuf }              (* Comments *)
| '('      { LPAREN }
| ')'      { RPAREN }
| '{'      { LBRACE }
| '}'      { RBRACE }
| ';'      { SEMI }
| ','      { COMMA }

(* binary operations *)
| '+'      { PLUS }
| '-'      { MINUS }
| '*'      { TIMES }
| '/'      { DIVIDE }
| '='      { ASSIGN }

(*arithmetic operations *)
| "=="     { EQ }
| "!="     { NEQ }
| '<'      { LT }
| "<="     { LEQ }
| ">"      { GT }
| ">="     { GEQ }

(*control flow *)
| "if"     { IF }
| "else"   { ELSE }
| "for"    { FOR }
| "while"  { WHILE }
| "return" { RETURN }
| "print" {PRINT}

| "int"    { INT }
| "Point"  { POINT }
| "Line"   { LINE }
| "Ellipse"{ ELLIPSE }
| "Curve"  { CURVE }
| "string" {STRING}
| "void"   {VOID}
| "boolean" {BOOLEAN}
| ".draw()" {DRAW}
| "MEllipse" {MELLIPSE}
| "MLine" {MLINE}
| "MPoint" {MPOINT}
| "MCurve" {MCURVE}
| ".getX()" {GETX}
| ".getY()" {GETY}


| ['0'-'9']+ as lxm { INTLITERAL(int_of_string lxm) }
| ['\"'] ([^ '\"']* as lit) ['\"'] { STRINGLITERAL(lit) }
| ['a'-'z' 'A'-'Z']['a'-'z' 'A'-'Z' '0'-'9' '_']* as lxm { ID(lxm) }
| eof { EOF }
| _ as char { raise (Failure("illegal character " ^ Char.escaped char)) }

and comment = parse
  "*/" { token lexbuf }  (* End-of-comment *)
| _    { comment lexbuf }  (* Eat everything else *)


